% Code for Generalized HKL Clustered Multi-task Learning (gHKL-CMTL) 
% by Pratik Jawanpuria, J. Saketha Nath and Ganesh Ramakrishnan (IIT Bombay)

% This code is built upon the code of HKL (Francis Bach: Exploring Large Feature
% Spaces with Hierarchical Multiple Kernel Learning. Advances in Neural
% Information Processing Systems (NIPS), 2008) available at
% http://www.di.ens.fr/~fbach/hkl/index.html
% Some of the functions are used as it is from above and some of them are
% discarded/modified.

% This code is licensed under GPL licence and it is free for non-commercial use. 


clear all
clc

%%%%%%PARAMETERS%%%%%%%%
mltask = 'classification';%classification, regression (classification - hinge loss, regression - epsilon regression)
weights = [1 1.5];% 1st is the weight of TOP node, second is the parameter 'a' such that d_v=a^|v|
rhonorm = 1.1;%(1<rhonorm<=2)
mu = .01;
maxactive = 10000;
high = 3;
low = -3;
inverted = false;%false;true;
errType = 0;%0: error, 1: (1-AUC), 2: MSE, 3: NMSE, 4: AMSE
balanceDataset = 1;
%%%%%%PARAMETERS%%%%%%%%

% LOAD DATA FILE containing feature matrix X and label vector Y
% X: n x d matrix where n is number of data points and d is dimension.
% Y is a n x 1 vector (for classification {-1,1} vector)

range = high - low + 1;
C = logspace(low,high,range);

% Prepare the train-test split - XTrain, YTrain, XTest, YTest

% numTrain : a kx1 vector where numTrain(i) = ni and ni represent the number of instance in i-th task.
% format of yTrain - (n1+n2+n3+...+nk)x1 vector.
% Ktr(x_{t_1i},x_{t_2j}) = k(x_{t_1i},x_{t_2j})*(1+1/mu) if t_1==t_2
%                        = k(x_{t_1i},x_{t_2j})/mu       otherwise
% here k is a kernel function - like linear/polynomial/gaussian etc
% Indices (sum(numTrain(1:(i-1)))+1):(sum(numTrain(1:i))) in both Ktr and YTrain should belong to i-th task.

% Similar procedure for test data: Kte yTest and (numTest)

[outputs,model,accuracies] = htl(Ktr,yTrain,Kte,yTest,C,weight,rhonorm,numTrain,numTest,...
    mu,'maxactive',maxactive,'inverted',inverted,'mltask',mltask,'balance',balanceDataset);

