function [outputs,dataset,accuracies] = htl(K_tr, Y_tr, K_te, Y_te, C, weights, rho, numTrain, numTest,mu, varargin)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% required parameter
% K - cell of size Txk. K{t,j} - must be PD
% Y - cell of size Tx1. y{t} is column vec. of labels for `t'^{th} task
% trInd - cell of size Tx1. trInd{t} is column vec. of training datapoint indices into gram-matrices in K
% C - common regularization parameter
% weights - weights [root weight, d_v, a_v]
% kerNorm - norm over kernels
% descNorm - norm over descendants
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

eta_hull = [];
alphaStruct = [];
hull = [];
inverted = false;
balance = 0;
mltask = 'regression';
% Y_te = [];
% av_change = 2;
% READ OPTIONAL PARAMETERS
args = varargin;
argstopass = []; % arguments to pass to later functions (solve_hkl)
nargs = length(args);
for i=1:2:nargs
	switch args{i},
		case 'display',         display = args{i+1}; argstopass = [ argstopass, i, i+1];
% 		case 'Xtest',           Xtest = args{i+1};
% 		case 'Ytest',           Ytest = args{i+1};
% 		case 'mingap',          mingap = args{i+1};  argstopass = [ argstopass, i, i+1];
		case 'maxactive',       maxactive = args{i+1};  argstopass = [ argstopass, i, i+1];
		case 'inverted',       inverted = args{i+1};  argstopass = [ argstopass, i, i+1];
		case 'mltask',       mltask = args{i+1};%  argstopass = [ argstopass, i, i+1];
		case 'balance',       balance = args{i+1};%  argstopass = [ argstopass, i, i+1];
% 		case 'data_normalization',            data_normalization = args{i+1};
% 		case 'kernel_normalization',            kernel_normalization = args{i+1};
		case 'hull',            hull = args{i+1};
% 		case 'alpha',        	alpha = args{i+1};
% 		case 'b',				b = args{i+1};
		case 'eta_hull',				eta_hull = args{i+1}; argstopass = [ argstopass, i, i+1];
		case 'av_change',				av_change = args{i+1};   argstopass = [ argstopass, i, i+1];         
% 		case 'memory_cache',   	memory_cache = args{i+1};
% 		case 'rhonorm',   	rhonorm = args{i+1}; argstopass = [ argstopass, i, i+1];
% 		case 'conflictMatrix',   	conflictMatrix = args{i+1}; argstopass = [ argstopass, i, i+1];            
% 		case 'gapeta',        	gapeta = args{i+1};  argstopass = [ argstopass, i, i+1];
% 		case 'conjgrad',        	conjgrad = args{i+1};  argstopass = [ argstopass, i, i+1];
	end
end
argstopass = args(argstopass);

T = length(Y_tr);
data.T = T; % Number of tasks - same as - dimension of the grid
% data.k = size(K,2); % Number of kernels
data.qs = 2*ones(1,T); % number of Tasks in each direction of the lattice. Used for traversing the lattice.
data.weight0 = weights(1); % weight of root
data.weightA = weights(2); % d_v : weight of 1st level nodes
% data.weightB = weights(3); % a_v : weight of 1st level nodes


wpos = zeros(T,1);
wneg = zeros(T,1);
if strcmp(mltask,'regression')
    balance = 0;
end

for t = 1:T
    if ~isempty(Y_te)
%         numTest(t) = length(Y_te{t});
    end
    if (balance==1)
        wpos(t) = sum(Y_tr{t} == -1)/numTrain(t);
        wneg(t) = sum(Y_tr{t} == 1)/numTrain(t);
        if (wpos(t)+wneg(t) ~= 1)
            fprintf('smo-htl - Positive and Negative examples not summing to 1\n');
            keyboard;
        end
    else
        wpos(t) = 1;
        wneg(t) = 1;
    end
end

dataset.mltask = mltask;
dataset.T = T;
dataset.numTrain = numTrain;
dataset.numTest = numTest;
dataset.wpos = wpos;
dataset.wneg = wneg;
dataset.balance = balance;
accuracies = cell(1,length(C));
outputs = cell(1,length(C));
for ilambda = 1:length(C)
    fprintf('*****************HTL- %f C\n',C(ilambda));
        output = solve_htl(K_tr,Y_tr,C(ilambda),rho,data,dataset,mu,'alpha',alphaStruct,'eta_hull',eta_hull, 'hull', hull,argstopass{:});
    fprintf('Testing for: %f descNorm - %f C\n\n',rho,C(ilambda));

    alphaStruct = output.alphaStruct;
	hull = output.hull;
	b_t = output.b_t;
	eta_hull = output.eta_hull;
	zeta_hull = output.zeta_hull;
	theta_hull = output.theta_hull;
	outputs{ilambda} = output;
%     if ~isempty(Y_te)
        [predtrain_t,predtest_t] = predict_htl(K_tr,K_te,hull,alphaStruct,b_t,zeta_hull,theta_hull,dataset,inverted,mu);
        % predtrain_t - cell of size Tx1. predtrain_t{t} is column vec. of labels for training points of `t'^{th} task
        % predtest_t - cell of size Tx1. predtest_t{t} is column vec. of labels for test points of `t'^{th} task
		accuracies.predtest{ilambda} = predtest_t;
		accuracies.predtrain{ilambda} = predtrain_t;
        if ~isempty(Y_te)
            if strcmp(mltask,'classification')
                taskwiseTrainErr = zeros(data.T,1);
                taskwiseTrainAuc = zeros(data.T,1);
                taskwiseTestErr = zeros(data.T,1);
                taskwiseTestAuc = zeros(data.T,1);
                for t = 1:data.T
                    trainPredicted = predtrain_t((sum(numTrain(1:(t-1)))+1):sum(numTrain(1:t)));
                    taskwiseTrainErr(t) = calculateErr(trainPredicted, Y_tr{t},'class_error');
                    taskwiseTrainAuc(t) = 1-calculateErr(trainPredicted, Y_tr{t},'auc');

                    testPredicted = predtest_t( (sum(numTest(1:(t-1)))+1):sum(numTest(1:t)));
                    taskwiseTestErr(t) = calculateErr(testPredicted, Y_te{t},'class_error');
                    taskwiseTestAuc(t) = 1-calculateErr(testPredicted, Y_te{t},'auc');
                end
                accuracies.taskwiseTrainErr(ilambda,:) = taskwiseTrainErr;
                accuracies.avgTrainErr(ilambda) = mean(taskwiseTrainErr);
                accuracies.taskwiseTrainAuc(ilambda,:) = taskwiseTrainAuc;
                accuracies.avgTrainAuc(ilambda) = mean(taskwiseTrainAuc);
                
                accuracies.taskwiseTestErr(ilambda,:) = taskwiseTestErr;
                accuracies.avgTestErr(ilambda) = mean(taskwiseTestErr);
                accuracies.taskwiseTestAuc(ilambda,:) = taskwiseTestAuc;
                accuracies.avgTestAuc(ilambda) = mean(taskwiseTestAuc);
            elseif strcmp(mltask,'regression')
                taskwiseTrainMse = zeros(data.T,1);
                taskwiseTrainNmse = zeros(data.T,1);
                taskwiseTrainAmse = zeros(data.T,1);
                
                taskwiseTestMse = zeros(data.T,1);
                taskwiseTestNmse = zeros(data.T,1);
                taskwiseTestAmse = zeros(data.T,1);
                for t = 1:data.T
                    trainPredicted = predtrain_t((sum(numTrain(1:(t-1)))+1):sum(numTrain(1:t)));
                    taskwiseTrainMse(t) = calculateErr(trainPredicted, Y_tr{t},'mse');
                    taskwiseTrainNmse(t) = calculateErr(trainPredicted, Y_tr{t},'nmse');
                    taskwiseTrainAmse(t) = calculateErr(trainPredicted, Y_tr{t},'amse');

                    testPredicted = predtest_t((sum(numTest(1:(t-1)))+1):sum(numTest(1:t)));
                    taskwiseTestMse(t) = calculateErr(testPredicted, Y_te{t},'mse');
                    taskwiseTestNmse(t) = calculateErr(testPredicted, Y_te{t},'nmse');
                    taskwiseTestAmse(t) = calculateErr(testPredicted, Y_te{t},'amse');
                end
                accuracies.taskwiseTrainMse(ilambda,:) = taskwiseTrainMse;
                accuracies.avgTrainMse(ilambda) = mean(taskwiseTrainMse);
                accuracies.taskwiseTrainNmse(ilambda,:) = taskwiseTrainNmse;
                accuracies.avgTrainNmse(ilambda) = mean(taskwiseTrainNmse);
                accuracies.taskwiseTrainAmse(ilambda,:) = taskwiseTrainAmse;
                accuracies.avgTrainAmse(ilambda) = mean(taskwiseTrainAmse);
                
                accuracies.taskwiseTestMse(ilambda,:) = taskwiseTestMse;
                accuracies.avgTestMse(ilambda) = mean(taskwiseTestMse);
                accuracies.taskwiseTestNmse(ilambda,:) = taskwiseTestNmse;
                accuracies.avgTestNmse(ilambda) = mean(taskwiseTestNmse);
                accuracies.taskwiseTestAmse(ilambda,:) = taskwiseTestAmse;
                accuracies.avgTestAmse(ilambda) = mean(taskwiseTestAmse);
            end
        end
%     end
%     keyboard;
end

% clearvars -global K_tr K_te Y_tr Y_te
