function [alphaOpt,bOpt,eta,omegaSquared,finalLpDualObj,theta,zeta,aTQa_t1t2] = ...
    mirrorDescentWrapper(K_tr,alpha_smo,y,nodeIdx,C,rho,eta,data,dataset,inverted,mu,varargin)
% mirrorDescentWrapper(alphaStruct,K_tr,Y_tr,C,kerNorm,descNorm,eta,data,dataset)

tube_eps = .1;
myRidge = 1e-8;
args = varargin;
nargs = length(args);
theta_flag = 0;
for i=1:2:nargs
    switch args{i},
        case 'final', theta_flag = 1;
    end
end

% solve the milasso for a given regularization parameter
mltask = dataset.mltask;
numTrain = dataset.numTrain;
T = length(numTrain);
p =length(data.affinity);
affs = data.affinity;
weights = data.weights;
hull = data.hull;
if inverted
    hull = 3-hull;
end
singleTaskMuFactor = 1+1/mu;
% traceFactor = zeros(p,1);
% for i=1:p
%     traceFactor(i) = 1;sum(traceVector(hull(i,:)==2));
% end

n = sum(numTrain);
fprintf('\n[mirrorDescentWrapper] No. of nodes: %d\n',p);
Y = [];
for t=1:T
    Y = [Y;y{t}];
end

lambda = 1/C;
%Calculate Kernel
myEps = 1e-6;
% READ OPTIONAL PARAMETERS

tic;
if (p >= 1)
    if (rho ~= 2)
        rhobar = rho/(2-rho);% MKL norm in primal
        rhobarstar = rhobar/(rhobar-1);
    else
        rhobar = inf;% MKL norm in primal
        rhobarstar = 1;
    end
    ITER_MAX = 500;

%     % WARM START
    if isempty(eta)
        eta = ones(p,1)/p;
    elseif ~all(eta)
        eta = eta + 1e-8;
    end
    
    eta = eta/sum(eta);

    % MIRROR DESCENT
    errCount = 0;
    dualObj = 1.7977e+300;
    min_alpha = nan;
    min_b = nan;
    min_eta = nan;
    min_zeta = nan;
    min_dualObj = 1.7977e+300;
    min_delta = nan;
    min_aTQa_smo = nan;
    
    for iter_gamma = 1:ITER_MAX
        % Solve for alpha ;given eta
        dualObjOld = dualObj;
        if (sum(isnan(eta)))
            fprintf('eta is NAN\n');
            keyboard;
        end
        fprintf('.');
        zetaFake = calculateZetaFake(affs,rho,weights,eta);
        zeta = nthroot(zetaFake,rhobarstar);
%         nonZero = zetaFake>0;
        if (rho ~= 2)
            tic
            fprintf('@');
            L_p = rhobar;
            switch (mltask)
                case {'classification'}
                    %% SMO for classification
                    if iter_gamma == 1
                        [alpha_smo,dualObj,aTQa_smo,bOpt] = smo_lpMKL_multiTask_K_mu(K_tr,nodeIdx,zeta,hull,Y,abs(alpha_smo),1/lambda,rhobar,dataset,0,mu);
                    else
                        [alpha_smo,dualObj,aTQa_smo,bOpt] = smo_lpMKL_multiTask_K_mu(K_tr,nodeIdx,zeta,hull,Y,alpha_smo,1/lambda,rhobar,dataset,0,mu);
                    end
                    alphaOpt = alpha_smo.*Y;
                    delta = 2*(sum(abs(alphaOpt)) - dualObj);
                     %% SMO for classification
                case {'regression'}
                    %% eps-SVR-SMO
                    [alpha_smo,dualObj,aTQa_smo,bOpt] = smo_lpMKL_reg_multiTask_K_mu(K_tr,nodeIdx,zeta,hull,Y,alpha_smo,1/lambda,L_p,dataset,tube_eps,mu);
                    alphaOpt = alpha_smo;
                    delta = 2*(Y'*alphaOpt - tube_eps*sum(abs(alphaOpt)) - dualObj);
%                     [alpha_cvx dualObj1] = cvxLpMklRegSmoMultitask_K_mu(K_tr,nodeIdx,zeta,hull,Y,1/lambda,rhobar,dataset,tube_eps,mu);
%                     (dualObj1-dualObj)/dualObj
%                     keyboard;
                    %% eps-SVR-SMO
                otherwise
                    fprintf('\n[mirrorDescentWrapper] Unknown mltask type\n');
                    keyboard;
            end
            fprintf('@');
            toc
            fprintf('\nDualObj: %f\n',dualObj);
%             keyboard;
        else
            % for rho=2, only one effective kernel is needed
            k = T*(T+1)/2;
            coeff = zeros(k,1);
            for i=1:p
                presentTasks = find(hull(i,:)==2);
                if (length(presentTasks) == 1)
                    coeff(presentTasks) = coeff(presentTasks) + zeta(i)/singleTaskMuFactor;
                else
                    pairs = combntns(presentTasks,2);
                    index = T + (pairs(:,1)-1).*(T-pairs(:,1)/2) + pairs(:,2) - pairs(:,1);
                    coeff([presentTasks';index]) = coeff([presentTasks';index]) + zeta(i);
                end
            end
            K2 = zeros(n,n);
            for t1=1:T
                for t2=t1:T
                    if t1 == t2
                        index3 = t1;
                        index1 = (sum(numTrain(1:(t1-1)))+1):sum(numTrain(1:t1));
                        K2(index1,index1) = K2(index1,index1) + K_tr(index1,index1)*coeff(index3);
                    else
                        index3 = T + (t1-1)*(T-t1/2) + t2 - t1;
                        index1 = (sum(numTrain(1:(t1-1)))+1):sum(numTrain(1:t1));
                        index2 = (sum(numTrain(1:(t2-1)))+1):sum(numTrain(1:t2));
                        Ktemp = K_tr(index1,index2)*coeff(index3);
                        K2(index1,index2) = K2(index1,index2) + Ktemp;
                        K2(index2,index1) = K2(index2,index1) + Ktemp';
                    end
                end
            end
            
%             for w=1:k
%                 K2 = K2+full(K{w}*coeff(w));
%             end
            nodeIdx_temp = true(n,1);% do not use ones(n,1)
            hull_temp = 2*ones(1,T);
            zeta_temp = 1;
            Lp_temp = 2;
            tic
            fprintf('@');
            switch (mltask)
                case {'classification'}
                    %% SMO for classification
                    if iter_gamma == 1
                        [alpha_smo,dualObj,aTQa_smo_fake,bOpt] = smo_lpMKL_multiTask_K_mu(K2,nodeIdx_temp,zeta_temp,hull_temp,Y,abs(alpha_smo),1/lambda,Lp_temp,dataset,0,mu);
                    else
                        [alpha_smo,dualObj,aTQa_smo_fake,bOpt] = smo_lpMKL_multiTask_K_mu(K2,nodeIdx_temp,zeta_temp,hull_temp,Y,alpha_smo,1/lambda,Lp_temp,dataset,0,mu);
                    end
                    alphaOpt = alpha_smo.*Y;
                    delta = 2*(sum(abs(alphaOpt)) - dualObj);
                     %% SMO for classification
                case {'regression'}
                    %% eps-SVR-SMO
                    [alpha_smo,dualObj,aTQa_smo_fake,bOpt] = smo_lpMKL_reg_multiTask_K_mu(K2,nodeIdx_temp,zeta_temp,hull_temp,Y,alpha_smo,1/lambda,Lp_temp,dataset,tube_eps,mu);
                    alphaOpt = alpha_smo;
                    delta = 2*(Y'*alphaOpt - tube_eps*sum(abs(alphaOpt)) - dualObj);
                    %% eps-SVR-SMO
                otherwise
                    fprintf('\n[mirrorDescentWrapper] Unknown mltask type\n');
                    keyboard;
            end
            fprintf('@');
            toc
            fprintf('\nDualObj: %f\n',dualObj);
            aTQa_smo = [];
            
%             aTQa_smo = nan(p,1);
%             for w=1:p
%                 aTQa_smo(w) = (alphaOpt(nodeIdx(:,w))'*K_tr(nodeIdx(:,w),nodeIdx(:,w))*alphaOpt(nodeIdx(:,w)) + (alphaOpt'*alphaOpt)*myRidge)*zeta(w);
%             end

%             [alpha_cvx dualObj1] = cvxLpMklRegSmoMultitask_K(K_tr,nodeIdx,zeta,hull,Y,1/lambda,rhobar,dataset,tube_eps);
%             fprintf('\nrel obj gap: %e\n',abs(dualObj1-dualObj)/dualObj);
%             if abs(dualObj1-dualObj)/dualObj > 1e-6
%                 keyboard;
%             end

%             % CVX begin
%             z = Y';
%             cvx_begin
%                 cvx_solver sdpt3;
%                 variable alpha_cvx(sum(numTrain),1);
%                 variable beta_cvx(sum(numTrain),1);
%                 
%                 maximize z*alpha_cvx - tube_eps*sum(beta_cvx) - 0.5*transpose(alpha_cvx)*K2*alpha_cvx
%                 subject to
%                     alpha_cvx + beta_cvx >= 0;
%                     beta_cvx - alpha_cvx >= 0;
%                     alpha_cvx + beta_cvx <= 2*1/lambda;
%                     beta_cvx - alpha_cvx <= 2*1/lambda;
%                     for t=1:T
%                         sum(alpha_cvx( (sum(numTrain(1:t-1))+1):(sum(numTrain(1:t))) )) >= 0;
%                         sum(alpha_cvx( (sum(numTrain(1:t-1))+1):(sum(numTrain(1:t))) )) <= 0;
%                     end
%             cvx_end
%             keyboard;

%             p1 = [(Y-tube_eps);(-Y-tube_eps)]';
%             Q = [[K2, -K2];[-K2', K2]];
%             Q = Q+eye(size(Q))*myRidge;
%             cvx_begin
%                 variable alpha_cvx1(2*n);
%                 variable tt;
% 
%                 maximize p1*alpha_cvx1 - 0.5*tt
% 
%                 subject to
%                     alpha_cvx1 >= 0;
%                     alpha_cvx1 <= 1/lambda;
%                     for t=1:T
%                         sum(alpha_cvx1( (sum(numTrain(1:t-1))+1):(sum(numTrain(1:t))) )) - ...
%                             sum(alpha_cvx1( (n+sum(numTrain(1:t-1))+1):(n+sum(numTrain(1:t))) )) >= 0;
%                         sum(alpha_cvx1( (sum(numTrain(1:t-1))+1):(sum(numTrain(1:t))) )) - ...
%                             sum(alpha_cvx1( (n+sum(numTrain(1:t-1))+1):(n+sum(numTrain(1:t))) )) <= 0;
%                     end
%                         tt >= alpha_cvx1'*Q*alpha_cvx1;
%             cvx_end
%             
%             keyboard;
%             alphaOpt = alpha_cvx.*Y;
%             dualObj = (sum(abs(alphaOpt)) - .5*alphaOpt'*K2*alphaOpt);
        end

        % compute a_iY_i Kij Y_ja_j for all task i,j
        aTQa_t1t2 = zeros(T*(T+1)/2,1);
        for t1=1:T
            for t2 = t1:T
                index1 = (sum(numTrain(1:(t1-1)))+1):sum(numTrain(1:t1));
                index2 = (sum(numTrain(1:(t2-1)))+1):sum(numTrain(1:t2));
                if t1 == t2
                    index3 = t1;
                    aTQa_t1t2(index3) = alphaOpt(index1)'*K_tr(index1,index2)*alphaOpt(index2);
                else
                    index3 = T + (t1-1)*(T-t1/2) + t2 - t1;
                    aTQa_t1t2(index3) = 2*(alphaOpt(index1)'*K_tr(index1,index2)*alphaOpt(index2));
                end
            end
        end
        
        if isempty(aTQa_smo)
            aTQa_smo = nan(p,1);
            for i=1:p
                presentTasks = find(hull(i,:)==2);
                if (length(presentTasks) == 1)
                    aTQa_smo(i) = aTQa_t1t2(presentTasks)/singleTaskMuFactor;
                else
                    pairs = combntns(presentTasks,2);
                    index = T + (pairs(:,1)-1).*(T-pairs(:,1)/2) + pairs(:,2) - pairs(:,1);
                    aTQa_smo(i) = sum(aTQa_t1t2(presentTasks)) + sum(aTQa_t1t2(index));
                end
            end
            aTQa_smo = (aTQa_smo + (alphaOpt'*alphaOpt)*myRidge).*zeta;
        end

        if (delta <= 0)
            fprintf('[mirrorDescentWrapper] Delta is negative\n');
            keyboard;
        end
%         if ( 1e-6 <= (dualObj-dualObjOld)/dualObj)
%             fprintf('[mirrorDescentWrapper] This is bad! dualObjOld=%e, dualObj=%e, rel.gap=%e\n',dualObjOld, dualObj,(dualObj-dualObjOld)/dualObj);
%         end
        if ( 1e-6 <= (dualObj-dualObjOld)/dualObj)
            fprintf('[mirrorDescentWrapper] This is bad! dualObjOld=%e, dualObj=%e, rel.gap=%e\n',dualObjOld, dualObj,(dualObj-dualObjOld)/dualObj);
            if errCount > 5
                fprintf('[mirrorDescentWrapper] errCount exceeds limit of 5. breaking MD\n');
                break;
            else
                errCount = errCount+1;
            end
        else
            errCount = 0;
        end

        %book-keeping
        if (dualObj < min_dualObj)
            min_alpha = alphaOpt;
            min_b = bOpt;
            min_eta = eta;
            min_zeta = zeta;
            min_dualObj = dualObj;
            min_delta = delta;
            min_aTQa_smo = aTQa_smo;
            min_aTQa_t1t2 = aTQa_t1t2;
        end
        %book-keeping
        if (abs((dualObjOld-dualObj)/dualObj) < myEps)
            break;
        end
        
        T2 = delta^rhobarstar;
        [gradHelp flag] = calculateSumForDesc(aTQa_smo,affs,rho,weights,eta,zetaFake);
        if flag
            fprintf('[mirrorDescentWrapper] flag of [calculateSumForDesc] is true\n');
            keyboard;
        end
        gradF = -(delta/T2)/(2*rhobarstar)*gradHelp;%alphaOpt,affs,rho,weights,eta,K,zetaFake,hull,T
        step = sqrt(log(p))/(sqrt(iter_gamma)*norm(gradF,inf));
        pvec = 1+log(eta) - step*gradF;
        eta = exp(pvec);
        eta = eta/sum(eta);
    end

    if (iter_gamma == ITER_MAX)
        fprintf('[mirrorDescentWrapper] *****Before convergence termination of MD*****, diff = %e\n',abs((dualObjOld-dualObj)/dualObj));
    end
    
    alphaOpt = min_alpha;
    bOpt = min_b;
    eta  = min_eta;
    zeta = min_zeta;
    dualObj = min_dualObj;
    delta = min_delta;
    aTQa_smo = min_aTQa_smo;
    aTQa_t1t2 = min_aTQa_t1t2;
    
    finalLpDualObj = dualObj;
    % Theta Computation
    if theta_flag % WRONG - debug for rho~=2 (zeta missing)
        theta = (aTQa_smo/delta).^(1/(rhobar-1));
    else
        theta = [];
    end
    omegaSquared = delta;
end

% [b_cvx dualObj_cvx] = cvxLpNormMdSVRSolver(K,Y,rho,C);
% keyboard;
% % for p==1 use CVX
% if (p == 1)
%     [alphaOpt,bOpt,eta,omegaSquared,finalLpDualObj,theta,zeta,aTKa] = cvxLpNormSolver(eta,data,Y,lambda,rho,K);
% end

fprintf('[mirrorDescentWrapper] MDiterations: %d, MDtime: ',iter_gamma);
toc;

% omegaSquared = omegaSquared*lambda/2;
% finalLpDualObj = finalLpDualObj;
fprintf('[mirrorDescentWrapper] Objective = %e\n',finalLpDualObj);
% keyboard;
%%%END function solve_hkl_hinge_shogun_pratik

function [zetaFake] = calculateZetaFake(affs,rho,weights,eta)
p =length(eta);
mat = zeros(p,p);
for i=1:p
    mat(i,affs{i}) = 1;
end
dffs = cell(p,1);
for i=1:p
    dffs{i} = find(mat(:,i)==1);
end

weightRho = weights.^(rho/(1-rho));
zetaFake = zeros(p,1);
etaFake = weightRho.*eta;
for i=1:p
    zetaFake(i,1) = norm(etaFake(dffs{i}),1-rho);
end


function [answer flag] = calculateSumForDesc(aTQa_smo,affs,rho,weights,eta,zetaFake)

p =length(eta);
% etaRho = eta.^rho;
% weightRho = weights.^rho;

if (rho ~= 2)
    rhobar = rho/(2-rho);% MKL norm in primal
    rhobarstar = rhobar/(rhobar-1);
else
    rhobar = inf;% MKL norm in primal
    rhobarstar = 1;
end

zeroEta = (eta==0);

aTQa_smo = aTQa_smo./(zetaFake.^(1/rhobarstar));% aTQa_smo passed to the function has zetaFake in it.
aTKaZeta = (aTQa_smo.^(0.5/(rho-1))).*(zetaFake);

answer = zeros(p,1);
for i=1:p
    if zeroEta(i)
        answer(i) = sum( (aTKaZeta(affs{i})*weights(i)/eta(i)).^rho );
    else
        answer(i) = sum( (aTKaZeta(affs{i})*weights(i)/eta(i)).^rho );
    end
end
flag = 0;
if any( isnan(answer) | isinf(answer) )
    flag = 1;
    fprintf('[mirrorDescentWrapper] eta is zero\n');
    keyboard;
end

% aTKa = zeros(p,1);

% for i=1:T% for first T positions in the hull
%     presentTasks = find(hull(i,:)==2);
%     aTKa(i) = aTQa_smo(presentTasks);
% end
% for i=(T+1):p
%     presentTasks = find(hull(i,:)==2);
%     pairs = combntns(presentTasks,2);
%     index = T + (pairs(:,1)-1).*(T-pairs(:,1)/2) + pairs(:,2) - pairs(:,1);
%     for j=[presentTasks index']
%         aTKa(i) = sum(aTQa_smo(index));
%     end
% %     coeff([presentTasks';index]) = coeff([presentTasks';index]) + zeta(i);
% end

% for i=1:p% for first T positions in the hull
%     presentTasks = find(hull(i,:)==2);
%     if length(presentTasks)==1
%         aTKa(i) = aTQa_smo(presentTasks);
%     else
%         pairs = combntns(presentTasks,2);
%         index = T + (pairs(:,1)-1).*(T-pairs(:,1)/2) + pairs(:,2) - pairs(:,1);
% %         for j=[presentTasks index']
%         aTKa(i) = sum(aTQa_smo([presentTasks';index]));
% %         end
%     end
% %     coeff([presentTasks';index]) = coeff([presentTasks';index]) + zeta(i);
% end

% for i=1:p
%     aTKa(i) = alphaOpt'*K(:,:,i)*alphaOpt;
% end

% aTKaZeta = (aTKa.^(rhobarstar)).*(zetaFake.^(rho));
% 
% answer = zeros(p,1);
% for i=1:p
%     answer(i) = sum(aTKaZeta(affs{i}));
% end
% 
% for i=1:p
%     if (etaRho(i) > 0)
%         answer(i) = answer(i)*weightRho(i)/etaRho(i);
%     else
%         fprintf('ethRho IS ZERO\n');
%         keyboard;
%     end
% end
