function [alphaOpt, bOpt, dualObj] = simpleSVM(K_tr,alphaStruct,y,C,zeta,theta,data,dataset,inverted,mu)

tube_eps = .1;
% solve the milasso for a given regularization parameter
mltask = dataset.mltask;
numTrain = dataset.numTrain;
T = length(numTrain);
p =length(data.affinity);
hull = data.hull;
if inverted
    hull = 3-hull;
end
singleTaskMuFactor = 1+1/mu;

n = sum(numTrain);
fprintf('\n[simpleSVM] No. of nodes: %d\n',p);
Y = [];
for t=1:T
    Y = [Y;y{t}];
end

%Calculate Kernel

k = T*(T+1)/2;
coeff = zeros(k,1);
theta_zeta = theta.*zeta;
for i=1:p
    presentTasks = find(hull(i,:)==2);
    if (length(presentTasks) == 1)
        coeff(presentTasks) = coeff(presentTasks) + theta_zeta(i)/singleTaskMuFactor;
    else
        pairs = combntns(presentTasks,2);
        index = T + (pairs(:,1)-1).*(T-pairs(:,1)/2) + pairs(:,2) - pairs(:,1);
        coeff([presentTasks';index]) = coeff([presentTasks';index]) + theta_zeta(i);
    end
end

K2 = zeros(n,n);

for t1=1:T
    for t2=t1:T
        if t1 == t2
            index3 = t1;
            index1 = (sum(numTrain(1:(t1-1)))+1):sum(numTrain(1:t1));
            K2(index1,index1) = K2(index1,index1) + K_tr(index1,index1)*coeff(index3);
        else
            index3 = T + (t1-1)*(T-t1/2) + t2 - t1;
            index1 = (sum(numTrain(1:(t1-1)))+1):sum(numTrain(1:t1));
            index2 = (sum(numTrain(1:(t2-1)))+1):sum(numTrain(1:t2));
            Ktemp = K_tr(index1,index2)*coeff(index3);
            K2(index1,index2) = K2(index1,index2) + Ktemp;
            K2(index2,index1) = K2(index2,index1) + Ktemp';
        end
    end
end

nodeIdx_temp = true(n,1);% do not use ones(n,1)
hull_temp = 2*ones(1,T);
zeta_temp = 1;
Lp_temp = 2;
tic
fprintf('@');
switch (mltask)
    case {'classification'}
        %% SMO for classification
        [alpha_smo,dualObj,aTQa_smo_fake,bOpt] = smo_lpMKL_multiTask_K_mu(K2,nodeIdx_temp,zeta_temp,hull_temp,Y,abs(alphaStruct),C,Lp_temp,dataset,0,mu);
        alphaOpt = alpha_smo.*Y;
         %% SMO for classification
    case {'regression'}
        %% eps-SVR-SMO
        [alpha_smo,dualObj,aTQa_smo_fake,bOpt] = smo_lpMKL_reg_multiTask_K_mu(K2,nodeIdx_temp,zeta_temp,hull_temp,Y,alphaStruct,C,Lp_temp,dataset,tube_eps,mu);
        alphaOpt = alpha_smo;
        %% eps-SVR-SMO
    otherwise
        fprintf('\n[mirrorDescentWrapper] Unknown mltask type\n');
        keyboard;
end
fprintf('@');
toc
if (dualObj < 0)
    fprintf('[SimpleSVM] Dual Obj is negative\n');
    keyboard;
end
% dualObj = dualObj/C;
fprintf('[SimpleSVM] Objective after alpha comptutation: %f\n',dualObj)

