function  [alpha,dualObj,aTQa,b_t] = smo_lpMKL_multiTask_K_mu(K_tr,nodeIdx,zeta,hull,Y,alpha,C,L_p,details,yKy,mu)

% lpmkl(K1(:,:,nonZero),ones(length(nonZero),1),1/lambda,mkl_norm,Y,w1/n,w2/n);

% T=1 (single task)
% K_tr - 3-d matrix of size nxnxk. where n is number of training points and
% k is the number of kernels must be PD % YKY will be computed here depending on yKy
% Y - nx1 vector of labels (all tasks included)
% alpha - alpha of last iteration, nx1 zero vector by default (all tasks included)
% C - regularization parameter
% L_p - p norm in the primal of lp-mkl
% details - details of the run. It contains
% details.balance 1 => apply proper weighing on C, 0 => otherwise
% details.numTrain - Tx1 vector number of training instances
% details.wpos - Tx1 weight of positive Xi
% details.wneg - Tx1 weight of negative Xi
% yKy = 0 => Q = YK_trY (TODO), else if yKy = 1 => K_tr = YKY (nothing todo)

% nodeIdx - numInstances x k where k is number of kernels
display = 0;

myEps = 1e-3;
TAU = 1e-12;
DTAU = 1e-5;
obj_threshold = 0.1;
diff_threshold = 1e-5;
myRidge = 1e-8;
MAXSMOITER = 1000000;

hull = hull-1;
singleTaskMuFactor = 1+1/mu;
T = details.T;
k = size(nodeIdx,2);% number of kernels
numTrain = details.numTrain;
n = sum(numTrain);
begIdx = [1;cumsum(numTrain(1:end-1))+1];
endIdx = cumsum(numTrain(1:end));

wpos = details.wpos;
wneg = details.wneg;

% Norms
if (L_p == 1)
    L_q = inf;
else
    L_q = L_p/(L_p -1);
end

% Q = YK_trY
if (yKy==0)
    indPos = Y==1;
    indNeg = Y==-1;
    YY = ones(n,n);
    YY(indPos,indNeg) = -1;
    YY(indNeg,indPos) = -1;
    Q = bsxfun(@times,YY,K_tr);
end

Q = Q + myRidge*eye(size(Q));% 10Sep2010

% % check +ve definite, TO be removed in the final runs
% for w=1:k
%     if min(eig(Q(nodeIdx(:,w),nodeIdx(:,w)))) <= 0 || ~isreal(eig(Q(nodeIdx(:,w),nodeIdx(:,w))))
%         fprintf('[smo_lpMKL_multiTask_K_mu] Error: non +ve definite matrix\n');
%         keyboard;
%     end
% end

if (isempty(alpha))
    % cold start
    alpha = zeros(n,1);
    aTQa = zeros(k,1);
    alpha_sum = 0;
    T1 = 0;
    denom=0;
    numon = zeros(k,1);
    obj = 0;
    d = zeros(k,1);
    Qalpha = zeros(n,k);
else
    % warm start - for alpha & aTQa
    alpha_sum = sum(alpha);
    aTQa = zeros(k,1);
    Qalpha = zeros(n,k);
    for w=1:k
%         aTQa(w) = alpha'*Q(:,:,w)*alpha;%vectorize_quad_single(Q(:,:,w),alpha);% can be bettered for sparse matrix
        numTask = sum(hull(w,:)==1);
        if numTask == 1
            Qalpha(nodeIdx(:,w),w) = ( Q(nodeIdx(:,w),nodeIdx(:,w))*alpha(nodeIdx(:,w)) )*(zeta(w)/singleTaskMuFactor);
            Qalpha(not(nodeIdx(:,w)),w) = alpha(not(nodeIdx(:,w)))*zeta(w)*myRidge;
            aTQa(w) = alpha'*Qalpha(:,w);
        else
            Qalpha(nodeIdx(:,w),w) = ( Q(nodeIdx(:,w),nodeIdx(:,w))*alpha(nodeIdx(:,w)) )*zeta(w);
            Qalpha(not(nodeIdx(:,w)),w) = alpha(not(nodeIdx(:,w)))*zeta(w)*myRidge;
            aTQa(w) = alpha'*Qalpha(:,w);
        end
    end
%     [d normTerm] = calObj(aTQa,L_q);% d: kx1, normTerm: 1x1
    
    T1 = norm(aTQa,L_q);
    denom = T1^(L_q - 1);
    numon = aTQa.^(L_q - 1);
    d = numon/denom;% kx1 vector
    obj = 0.5*T1;
    obj = obj - alpha_sum;
    
%     Qalpha = shiftdim(alpha'*Q,1);% vector nxk
end

get_C = zeros(n,1);
idx_pos = (Y==1);
idx_neg = (Y==-1);
idx_t = cell(T,1);
for t=1:T
    idx = (begIdx(t):endIdx(t));
    idx_t{t} = idx;
    get_C(idx(idx_pos(idx))) = C*wpos(t);
    get_C(idx(idx_neg(idx))) = C*wneg(t);
end

I_low = ((alpha>0)&(Y == 1)) | ((alpha<get_C)&(Y == -1)); % 1-0 vectors
I_up = ((alpha>0)&(Y == -1)) | ((alpha<get_C)&(Y == 1)); % 1-0 vectors

% SMO starts

% while KKT not satisfied
% choose the most violating pair
% update the pair

% i_old =0;j_old=0;

smoIter = 0;
nrIter = 0;
nrIterLocalMaxRand = 100;
nrRandStart = 0;
bisectIter = 0;
i_old = nan;
j_old = nan;
bestT_old = nan;
repeatCounter = 0;
repeatCounterGlobal = 0;
repeatCounterMax = 1000;
randStartCounter = 0;

% newtonRaphsonIter = zeros(50,1);
% objVec = [];counter = 0;
gap = inf;
while (true)
    smoIter = smoIter+1;

    if display
        fprintf('SMO iteration = %d\n',smoIter);
    end

    if (mod(smoIter,1000)==0)
        fprintf('.');
%         keyboard;
    end
    if (mod(smoIter,50000)==0)
%         keyboard;
        fprintf('%f \n',obj);
%         disp(aTQa');
    end
    
    % CALCULATE gradient w.r.t. alpha - G - nx1 vector
    % Update G : g_from_Qalpha();
    G = Qalpha*d-1;
    
    % select working set
    %checkKKT()
    bestGap = -Inf;
    bestMaxIndex = nan;
    bestMinIndex = nan;
    bestT = nan;
    converge = true;
    yG = G.*Y;
    gapT = zeros(1,T);
    minIndexT = nan(1,T);
    maxIndexT = nan(1,T);
    for t=1:T
        iup = find(I_up(idx_t{t}))+begIdx(t)-1;
        ilow = find(I_low(idx_t{t}))+begIdx(t)-1;
        [maxVal max_i] = max( -yG(iup) );% max of -ve quantity = min of +ve quantity
        maxIndex = iup(max_i);
%         ygrad_low = -(Y(ilow)).*(G(ilow));
        [minVal min_j] = min( -yG(ilow) );% min of -ve quantity = max of -ve quantity
        minIndex = ilow(min_j);
        gap = (maxVal - minVal);
        gapT(t) = gap;
        minIndexT(t) = minIndex;
        maxIndexT(t) = maxIndex;
        if (gap > myEps)
            if gap > bestGap
                bestGap = gap;
                bestMaxIndex = maxIndex;
                bestMinIndex = minIndex;
                bestT = t;
                converge = false;
            end
        end
    end
%     if (mod(smoIter,10000)==0)
%         disp(gapT);
% 
%     end

    if converge
%         keyboard;
%         fprintf('smo-htl - SMO convergence\n')
        break;% SMO converged
    elseif (smoIter> MAXSMOITER/2) && (bestGap <= myEps*10)
        fprintf('[smo_lpMKL_multiTask_K_mu] - termination SMO - smoiter more than half-MAXSMOITER and gap sufficiently small\n');
        break;
    elseif smoIter> MAXSMOITER
        fprintf('[smo_lpMKL_multiTask_K_mu] - termination SMO - smoiter more than MAXSMOITER\n');
        break;
    end
    % main code
    % WSS1, i,j belong to same Task
    i = bestMaxIndex;% I_up
    j = bestMinIndex;% I_low
    
    if i==i_old && j==j_old && bestT==bestT_old
        repeatCounter = repeatCounter+1;
        repeatCounterGlobal = repeatCounterGlobal+1;
        if (repeatCounter > repeatCounterMax)
            fprintf('*');
%             fprintf('[smo_lpMKL_multiTask_K_allTaskReg] - Repeated indices greater than %d, doing random start\n',repeatCounterMax);
            violatingTask = find(gapT>myEps);
%             keyboard;
            while length(violatingTask)>1
                bestT1 = violatingTask(randi(length(violatingTask),1));
                if bestT1 ~= bestT
                    bestT = bestT1;
                    break;
                end
            end
            i = maxIndexT(bestT);
            j = minIndexT(bestT);
            if length(violatingTask)>1
                randStartCounter = randStartCounter+1;
            end
            repeatCounter = 0;
        end
%         if (repeatCounter > repeatCounterMax)
%             keyboard;
%         end
    else
        repeatCounter = 0;
    end
    i_old=i;j_old=j;bestT_old=bestT;
    
    Q_i = Q(:,i);
    Q_j = Q(:,j);
    C_i = get_C(i);
    C_j = get_C(j);
    old_alpha_i=alpha(i);
    old_alpha_j=alpha(j);

    %Find a direction of descent 
    rr = 2*myRidge*ones(1,k);
    presentNodes = find(hull(:,bestT));
    absentNodes = find(1-hull(:,bestT));
    singleTaskNode = presentNodes(sum(hull(presentNodes,:),2)==1);
    if length(singleTaskNode) > 1
        fprintf('[smo_lpMKL_multiTask_K_mu] number of singleTaskNode not equal to 1\n');
        keyboard;
    end

    if Y(i)==Y(j)
        if G(i)-G(j) < G(j) - G(i)
            dir_i=1;
            dir_j=-1;
            dir_grad=G(i) - G(j);
            dir_grad_offset=0;%p[i] - p[j];
            eta_max=min(C_i - alpha(i), alpha(j));
        else
            dir_i=-1;
            dir_j=1;
            dir_grad=G(j) - G(i);
            dir_grad_offset=0;%p[j] - p[i];
            eta_max=min(alpha(i), C_j - alpha(j));
        end
        % % 10Sep2010 rr(presentNodes)=(Q(i,i) + Q(j,j) - 2*Q(i,j) + 2*myRidge);
        rr(presentNodes)=(Q(i,i) + Q(j,j) - 2*Q(i,j));
    else
        if G(i)+G(j) < -G(j) - G(i)
            dir_i=1;
            dir_j=1;
            dir_grad=G(i) + G(j);
            dir_grad_offset=-2;%p[i] + p[j];
            eta_max=min(C_i - alpha(i), C_j - alpha(j));
        else
            dir_i=-1;
            dir_j=-1;
            dir_grad=-G(i) - G(j);
            dir_grad_offset=2;%-p[j] - p[i];
            eta_max=min(alpha(i), alpha(j));
        end
        % % 10Sep2010 rr(presentNodes)=(Q(i,i) + Q(j,j) + 2*Q(i,j) + 2*myRidge);
        rr(presentNodes)=(Q(i,i) + Q(j,j) + 2*Q(i,j));
    end
    rr(singleTaskNode) = rr(singleTaskNode)/singleTaskMuFactor;
    rr = rr.*zeta';
    
    % Compute directional Hessian
    % Newton - raphson
    nrIter = nrIter+1;% eta = 0 (eta has been used instead of delta)

    % obtain dir_hess
    if (T1==0)
        dir_hess = 0;
    else
        H1 = (1-L_q)*(4*(dir_grad-dir_grad_offset)^2)/T1;
%         H1 = (4*(dir_grad-dir_grad_offset)^2)/T1/(L_q-1);
        TH1 = (dir_i*Qalpha(i,:) + dir_j*Qalpha(j,:));% b_k
        H2 = (2*rr*d + 4*(L_q-1)*(TH1.^2)*(d./aTQa));% 4 is due to square(2b_k)
        dir_hess = 0.5*(H1+H2);
        if (dir_hess < 0)
            dir_hess = TAU;
%             fprintf('\n-ve hessian\n');
%             keyboard;
        end
    end
    
    if dir_hess <= 0
        dir_hess=TAU;
    end
    
    old_obj = obj;
    old_dir_grad = dir_grad;
    eta_min = 0;
    eta = -dir_grad/dir_hess;
    if(eta > eta_max)
        eta = eta_max;
    elseif eta < eta_min
        eta = 0.5*(eta_max + eta_min);
    end
%     if display
%         fprintf('Obj: %f, dir_grad: %f\n',obj,dir_grad);
%         alpha'
%         fprintf('1: %f, 2: %f\n',aTQa(1),alpha'*Q(:,:,1)*alpha);
%     end
    nrIterLocal = 0;
    while true
%         keyboard;
        oldAlpha = alpha;
%         counter = counter+1;
%         objVec(counter) = obj;
        new_alpha_i=old_alpha_i + eta*dir_i;
        new_alpha_j=old_alpha_j + eta*dir_j;
        change_i=new_alpha_i - alpha(i);
        change_j=new_alpha_j - alpha(j);      
        alpha(i)=new_alpha_i;
        alpha(j)=new_alpha_j;
        
        % update_Qalpha - i.e. aTQa
        r1 = myRidge*ones(k,1);
        r1(presentNodes) = Q(i,i);% % 10Sep2010 +myRidge;
        r1(singleTaskNode) = r1(singleTaskNode)/singleTaskMuFactor;
%         r1 = r1.*zeta;
        r2 = myRidge*ones(k,1);
        r2(presentNodes) = Q(j,j);% % 10Sep2010 +myRidge;
        r2(singleTaskNode) = r2(singleTaskNode)/singleTaskMuFactor;
%         r2 = r2.*zeta;
        r3 = zeros(k,1);
        r3(presentNodes) = Q(i,j);
        r3(singleTaskNode) = r3(singleTaskNode)/singleTaskMuFactor;
%         r3 = r3.*zeta;
%         if smoIter == 1056
%             keyboard;
%         end
%         % test aTQa components
%         delAlpha = zeros(n,1);
%         delAlpha(i) = change_i;
%         delAlpha(j) = change_j;
%         Comp1 = zeros(k,1);
%         Comp2 = zeros(k,1);
%         for w=1:k
%             Comp1(w) = (delAlpha(nodeIdx(:,w))'*Q(nodeIdx(:,w),nodeIdx(:,w))*delAlpha(nodeIdx(:,w)) +...
%                 myRidge*delAlpha(not(nodeIdx(:,w)))'*delAlpha(not(nodeIdx(:,w))))*zeta(w);
%             Comp2(w) = 2*(delAlpha(nodeIdx(:,w))'*Q(nodeIdx(:,w),nodeIdx(:,w))*oldAlpha(nodeIdx(:,w)) +...
%                 myRidge*delAlpha(not(nodeIdx(:,w)))'*oldAlpha(not(nodeIdx(:,w))))*zeta(w);
%         end
%         aTQa_check1 = aTQa + Comp1 + Comp2;
%         if smoIter > 1
%             aTQa_check3 = aTQa_check + Comp1 + Comp2;
%         end
%         aTQa_old = aTQa;
%         % test aTQa components
        aTQa = aTQa + ( r1*change_i*change_i + r2*change_j*change_j + 2*r3*change_i*change_j ).*zeta +...
            2*( Qalpha(i,:)*change_i+Qalpha(j,:)*change_j)'; %a*delta^2 + 2*b*delta;
        if any(aTQa < 0)
            fprintf('[smo_lpMKL_multiTask_K_mu] -ve aTQa\n');
            keyboard;
        end
        
%         aTQa_check = zeros(k,1);
%         for ii=1:k
%             aTQa_check(ii) =  ( (alpha(nodeIdx(:,ii)))'*Q(nodeIdx(:,ii),nodeIdx(:,ii))*alpha(nodeIdx(:,ii)) + ...
%             myRidge*(alpha(not(nodeIdx(:,ii)))'*alpha(not(nodeIdx(:,ii))) ) )*zeta(ii);
%         end
%         fprintf('aTQa norm diff: %d\n',norm(aTQa-aTQa_check)/norm(aTQa));
%         fprintf('aTQa norm diff: %d\n',norm(aTQa-aTQa_check1)/norm(aTQa));
%         fprintf('aTQa norm diff: %d\n\n',norm(aTQa_check1-aTQa_check)/norm(aTQa));
%         if (norm(aTQa-aTQa_check)/norm(aTQa) > 1e-3)
%             keyboard;
%         end
        
%         aTQa = zeros(k,1);
%         for ii=1:k
%             aTQa(ii) =  ( (alpha(nodeIdx(:,ii)))'*Q(nodeIdx(:,ii),nodeIdx(:,ii))*alpha(nodeIdx(:,ii)) + ...
%             myRidge*(alpha(not(nodeIdx(:,ii)))'*alpha(not(nodeIdx(:,ii))) ) )*zeta(ii);
%         end
%         fprintf('aTQa norm diff: %d\n',norm(aTQa-aTQa_check)/norm(aTQa));
%         if (norm(aTQa-aTQa_check)/norm(aTQa) > 1e-1)
%             keyboard;
%         end
        
        % update_Qalpha
        if ~isempty(absentNodes)
            % for absentNodes
            Qalpha(i,absentNodes) = Qalpha(i,absentNodes) + myRidge*change_i*zeta(absentNodes)';
            Qalpha(j,absentNodes) = Qalpha(j,absentNodes) + myRidge*change_j*zeta(absentNodes)';
        end
        for w=presentNodes'
            q1 = zeros(n,1);
            q2 = zeros(n,1);
            if w == singleTaskNode
                q1(nodeIdx(:,w)) = Q_i(nodeIdx(:,w))*change_i*(zeta(w)/singleTaskMuFactor);
                q2(nodeIdx(:,w)) = Q_j(nodeIdx(:,w))*change_j*(zeta(w)/singleTaskMuFactor);
            else
                q1(nodeIdx(:,w)) = Q_i(nodeIdx(:,w))*change_i*zeta(w);
                q2(nodeIdx(:,w)) = Q_j(nodeIdx(:,w))*change_j*zeta(w);
            end
            Qalpha(:,w) = Qalpha(:,w) + q1 + q2;
        end
        
%         Qalpha_check = zeros(n,k);
%         for w=1:k
%             Qalpha_check(nodeIdx(:,w),w) = Q(nodeIdx(:,w),nodeIdx(:,w))*alpha(nodeIdx(:,w))*zeta(w);
%             Qalpha_check(not(nodeIdx(:,w)),w) = myRidge*alpha(not(nodeIdx(:,w)))*zeta(w);
%         end
%         Qalpha = zeros(n,k);
%         for w=1:k
%             Qalpha(nodeIdx(:,w),w) = Q(nodeIdx(:,w),nodeIdx(:,w))*alpha(nodeIdx(:,w))*zeta(w);
%             Qalpha(not(nodeIdx(:,w)),w) = myRidge*alpha(not(nodeIdx(:,w)))*zeta(w);
%         end
%         fprintf('Qalpha norm diff: %d\n',norm(Qalpha-Qalpha_check)/norm(Qalpha));
%         if (norm(Qalpha-Qalpha_check)/norm(Qalpha) > 1e-7)
%             keyboard;
%         end
        
        
%         keyboard;
        
        alpha_sum = alpha_sum + change_i + change_j;
        T1 = norm(aTQa,L_q);
        denom = T1^(L_q - 1);
        numon = aTQa.^(L_q - 1);
        d = numon/denom;% kx1 vector

        obj = 0.5*T1;
        obj = obj - alpha_sum;
        
        % Compute new directional gradient
        dir_grad = dir_grad_offset;
        TH1 = (dir_i*Qalpha(i,:) + dir_j*Qalpha(j,:));% b_k
        largeOnes = d>DTAU;
        dir_grad = dir_grad + TH1(largeOnes)*d(largeOnes);
%         if display
%             fprintf('Obj: %f, change_i: %f, change_j: %f, dir_grad: %f\n',obj,change_i,change_j,dir_grad);
% %             alpha'
% %             fprintf('1: %f, 2: %f\n',aTQa(1),alpha'*Q(:,:,1)*alpha);
%         end

        if ( obj < old_obj + obj_threshold*eta*old_dir_grad ||...
                abs(eta_max - eta_min) < diff_threshold ||...
                abs(dir_grad) < diff_threshold ||...
                (eta == eta_max && dir_grad < 0) ||...
                (eta == eta_min && dir_grad > 0) )
            % Small problem convergence
            break;
        end
        if(dir_grad > 0)
            eta_max=eta;
        elseif(eta >= eta_min+TAU)
            eta_min=eta;
        end
        
        % Try a Newton-Raphson step
        % Compute directional Hessian
        
        % obtain dir_hess
%         H1 = -(0.25*(dir_grad-dir_grad_offset)^2)/T1/L_p;
        H1 = (1-L_q)*(4*(dir_grad-dir_grad_offset)^2)/T1;
%         H1 = (.25*(dir_grad-dir_grad_offset)^2)/T1/(L_q-1);
        H2 = (2*rr*d + 4*(L_q-1)*(TH1.^2)*(d./aTQa));% 4 is due to square(2b_k)
        dir_hess = 0.5*(H1+H2);

        if(dir_hess <= 0)
            dir_hess=TAU;
        end
        eta=eta - dir_grad/dir_hess;
        % If the result is outside the brackets, use bisection
        if(eta > eta_max || eta < eta_min)
            bisectIter = bisectIter+1;
            eta = 0.5*(eta_max + eta_min);
        end
        nrIter = nrIter+1;
        nrIterLocal = nrIterLocal+1;
        if mod(nrIterLocal,nrIterLocalMaxRand)==0
            nrRandStart = nrRandStart+1;
            eta = 0.5*(eta_max + eta_min);
        end

    end
%     keyboard;

    % update gradients w.r.t. alpha using d, Qalpha
    
%     newtonRaphsonIter(smoIter,1) = nrIter;
    
    if (display==1)
        fprintf('[smo_lpMKL_singleTask] Value of delta: %f\n',eta);
    end
    
    % update I_up I_low accordingly
    if (alpha(i)<C_i && Y(i)==1) || (alpha(i)>0 && Y(i)==-1)
        I_up(i) = 1;
    else
        I_up(i) = 0;
    end
    if (alpha(i)<C_i && Y(i)==-1) || (alpha(i)>0 && Y(i)==1)
        I_low(i) = 1;
    else
        I_low(i) = 0;
    end
    if (alpha(j)<C_j && Y(j)==1) || (alpha(j)>0 && Y(j)==-1)
        I_up(j) = 1;
    else
        I_up(j) = 0;
    end
    if (alpha(j)<C_j && Y(j)==-1) || (alpha(j)>0 && Y(j)==1)
        I_low(j) = 1;
    else
        I_low(j) = 0;
    end


    
    
    
%     % update the pair of alpha
%     i3 = i1;
%     if Y{t_v}(i3) == 1
%         C_i3 = C*wpos(t_v);
%     else
%         C_i3 = C*wneg(t_v);
%     end
%     if (alpha_t{t_v}(i3)+delta < myTol)
%         delta = -alpha_t{t_v}(i3);
%         %         x = delta;
%         alpha_t{t_v}(i3) = 0;
%     elseif (C_i3-alpha_t{t_v}(i3)-delta < myTol)
%         delta = C_i3-alpha_t{t_v}(i3);
%         %         x = delta;
%         alpha_t{t_v}(i3) = C_i3;
%     else
%         alpha_t{t_v}(i3) = alpha_t{t_v}(i3)+delta;
%     end
%     
%     i3 = i2;
%     if Y{t_v}(i3) == 1
%         C_i3 = C*wpos(t_v);
%     else
%         C_i3 = C*wneg(t_v);
%     end
%     if (alpha_t{t_v}(i3)+s*delta < myTol)
%         delta = -alpha_t{t_v}(i3)/s;
%         %         x = delta;
%         alpha_t{t_v}(i3) = 0;
%     elseif (C_i3-alpha_t{t_v}(i3)-s*delta < myTol)
%         delta = (C_i3-alpha_t{t_v}(i3))/s;
%         %         x = delta;
%         alpha_t{t_v}(i3) = C_i3;
%     else
%         alpha_t{t_v}(i3) = alpha_t{t_v}(i3)+s*delta;
%     end
    
end

% calculate b_t
b_t = zeros(T,1);
yG = G.*Y;
nr_free = alpha > 0 & alpha < get_C;
for t=1:T
    nr_free_task = nr_free(idx_t{t});
    yG_task = yG(idx_t{t});
    if any(nr_free_task)
        b_t(t) = sum(yG_task(nr_free_task))/sum(nr_free_task);
    else
        iup = find(I_up(idx_t{t}))+begIdx(t)-1;
        ilow = find(I_low(idx_t{t}))+begIdx(t)-1;
        maxVal = min(yG(iup));
        minVal = max(yG(ilow));
        b_t(t) = (minVal+maxVal)/2;
    end
end
b_t = -b_t;% following the decision model of libsvm
% calculate final objective
fprintf('[smo_lpMKL_multiTask_K_mu] gap: %e, smoIter: %d, nr: %d, bs: %d, randStart: %d, repeats: %d, nrRandStart: %d\n',max(gapT),smoIter,nrIter,...
    bisectIter,randStartCounter,repeatCounterGlobal,nrRandStart);
dualObj = -obj;
% aTQa_check = zeros(k,1);
% for i=1:k
%     aTQa_check(i) = ( (alpha(nodeIdx(:,i)))'*Q(nodeIdx(:,i),nodeIdx(:,i))*alpha(nodeIdx(:,i)) + myRidge*(alpha'*alpha) )*zeta(i);
% end
% fprintf('[smo_lpMKL_oneTask_K] - gap: %f, smoIter: %d, nr: %d, bs: %d\n',gap,smoIter,nrIter,bisectIter);

% % for testing correctness
% for t=1:T
%     for j=1:k
%         TestaTKa(t,j) = alpha_t{t}'*K_tr{t,j}*alpha_t{t};
%     end
% end
% for w=1:p
%     presentTasks = find(hull(w,:)==2);
%     for j=1:k
%         TestaTKSa(w,j) = sum(TestaTKa(presentTasks,j),1);
%     end
% end
% for w=1:p
%     TestBw(w) = nthroot(zeta_av_rhobarstar(w),rhobarstar)*norm(TestaTKSa(w,:),q);
% end
% for t=1:T
%     TestalphaSum(t) = sum(alpha_t{t});
% end
% TestdualObj = -0.5*norm(TestBw,rhobarstar) + sum(TestalphaSum);
%

% function [d obj] = calObj(aTQa,L_q)
%     obj = norm(aTQa,L_q);
%     denom = obj^(L_q - 1);
%     d = (aTQa.^(L_q - 1))/denom;% kx1 vector
%     obj = 0.5*obj;
